﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using ff14bot;
using Clio.Utilities;
using ff14bot.Helpers;
using ff14bot.Settings;
using ff14bot.Enums;
using ff14bot.Managers;
using Newtonsoft.Json;
using System.IO;

namespace Classy_1
{
    public partial class Form1 : Form
    {

        //Collection Binding for ComboBox
        #region ComboBox Binding Source

        //Add custom keybindings here
        //Note: Keybinds should be in the Keys. namespace.. Eg: Keys.X, Keys.Numpad0 ect..
        //Also, keybinds should be set in FFXIV, cooresponding to the list below
        //and mapped to gear sets for specific class.
        List<string> keys = new List<string>()
            {
                "None",
                "1",
                "2",
                "3",
                "4",
                "5",
                "6",
                "7",
                "8",
                "9",
                "10"
            };
        #endregion

        //Dictionary to hold the Job/Keybind to validate against when selecting other jobs/kebinds.
        //also is used to get the keybind value to send to the behaviour in CustomTag.cs
        public static Dictionary<String, String> bindings = new Dictionary<string, string>();



        public Form1()
        {
            InitializeComponent();



        }

        //called on Form Load to load the saved bindings and to display them in the combo boxes on Form1
        //IMO this is sloppy. There must be a better way to set all of these. I do not know much about databinding in winforms.
        //any help with this would be appreciated.
        public void SetSavedBindings()
        {
            if (File.Exists(Path.Combine(GlobalSettings.Instance.PluginsPath, @"Classy\Settings\binding.json")))
            {
                var json = File.ReadAllText(Path.Combine(GlobalSettings.Instance.PluginsPath, @"Classy\Settings\binding.json"));
                var obj = JsonConvert.DeserializeObject<Dictionary<string, string>>(json);
                foreach (KeyValuePair<string, string> item in obj)
                {

                    var c = Form1.GetAll(this, typeof(ComboBox));
                    foreach (Control x in c)
                    {

                        if (x.GetType() == typeof(ComboBox))
                        {

                            if (((ComboBox)x).Name == item.Key.ToString())
                            {

                                ((ComboBox)x).SelectedItem = item.Value.ToString();

                            }
                        }
                    }
                }
            }
            else
            {

            }
        }

        //Method to sort through all the controls on the form.
        //using this since controls in groupboxes are not considered as "being on the form"
        public static IEnumerable<Control> GetAll(Control control, Type type)
        {
            var controls = control.Controls.Cast<Control>();

            return controls.SelectMany(ctrl => GetAll(ctrl, type))
                                      .Concat(controls)
                                      .Where(c => c.GetType() == type);
        }

        //Just sets the item collections for the combo boxes based on the List<> setting where I'm keeping the keybind options.
        //Currently only the numpad is being used, since I personally do not use the numpad during actual gameplay.
        public void ReloadKeys()
        {
            var c = GetAll(this, typeof(ComboBox));
            foreach (Control x in c)
            {
                if (x.GetType() == typeof(ComboBox))
                {
                    ((ComboBox)x).BindingContext = new BindingContext();
                    ((ComboBox)x).DataSource = keys;
                }
            }
        }


        //Loads item collection into the combo boxes, also calls to set the SelectedItem value based off the saved json file located
        //in the Settings Directory.
        private void Form1_Load(object sender, EventArgs e)
        {
            ReloadKeys();
            SetSavedBindings();


        }

        //The only SelectedIndexChanged event for all 12 combo boxes.
        //The ComboBox name is used as the "Job" name.
        //This is making it difficult to do "White Mage" and "Black Mage" this way since a controll
        //name can not include a space.. maybe a case/switch when deciding?
        private void comboBox_SelectedIndexChanged(object sender, EventArgs e)
        {


            var comboBox = (ComboBox)sender;
            if (comboBox.SelectedItem.ToString() != "None")
            {
                //if the dictionary is at 0, add the first change without checking the binding dictionary for dupes
                if (bindings.Count == 0)
                {
                    bindings.Add(comboBox.Name, comboBox.SelectedItem.ToString());
                }

                    //display custom messagebox.cs informing a job is already bound to the dictionary
                else if (bindings.ContainsValue(comboBox.SelectedItem.ToString()))
                {
                    //using custom form for messagebox since the .net MessageBox could not easily be set to be in the middle of the 
                    //parent form.
                    messagebox msgbox = new messagebox();
                    msgbox.StartPosition = FormStartPosition.CenterParent;
                    msgbox.ShowDialog(this);
                    comboBox.SelectedIndex = 0;
                }

                        //check to see if you are changing a selection that had once already been keybound.
                //remove it from the dictionary and add the new selection.
                //??? maybe just change the value of the selected key? (last minute thought)
                else if (bindings.ContainsKey(comboBox.Name))
                {
                    bindings.Remove(comboBox.Name);
                    bindings.Add(comboBox.Name, comboBox.SelectedItem.ToString());
                }

                    //Job has not been listed to the dictionary.. go ahead and add the selection
                else
                {
                    bindings.Add(comboBox.Name, comboBox.SelectedItem.ToString());
                }


            }
            //"None" was selected as the keybind, so go ahead and remove the job key from the dictionary to free up the previous keybind.
            else
            {
                bindings.Remove(comboBox.Name);
            }


        }


        //Save Button.. saves the dictionary(bindings) as json to the Settings Folder
        private void button1_Click(object sender, EventArgs e)
        {
            string json = JsonConvert.SerializeObject(bindings, Formatting.None);
            File.WriteAllText(Path.Combine(GlobalSettings.Instance.PluginsPath, @"Classy\Settings\binding.json"), json);
            this.Close();

        }









    }
}
